@@ -6,6 +6,7 @@
 #include "gemmi/pdb.hpp"
 #include "gemmi/calculate.hpp"
 #include "gemmi/neighbor.hpp"
+#include "gemmi/select.hpp"  // count_atom_sites
 #include <benchmark/benchmark.h>
 
 static const char* path;
@@ -87,6 +88,56 @@ static void fractional_box(benchmark::State& state) {
   }
 }
 
+static bool has_hydrogen_with_levels(const gemmi::Structure& st) {
+  for (const gemmi::Model& model : st.models)
+    for (const gemmi::Chain& chain : model.chains)
+      for (const gemmi::Residue& res : chain.residues)
+        for (const gemmi::Atom& atom : res.atoms)
+          if (atom.is_hydrogen())
+            return true;
+  return false;
+}
+
+static bool has_hydrogen_with_cra(const gemmi::Structure& st) {
+  for (const gemmi::Model& model : st.models)
+    for (gemmi::const_CRA cra : model.all())
+      if (cra.atom->is_hydrogen())
+        return true;
+  return false;
+}
+
+static bool has_hydrogen_with_selection(const gemmi::Structure& st) {
+  gemmi::Selection sel("[H,D]");
+  return count_atom_sites(st, &sel) != 0;
+}
+
+static void has_hydrogen1(benchmark::State& state) {
+  using namespace gemmi;
+  Structure st = read_pdb_file(path);
+  while (state.KeepRunning()) {
+    bool has_hydr = has_hydrogen_with_levels(st);
+    benchmark::DoNotOptimize(has_hydr);
+  }
+}
+
+static void has_hydrogen2(benchmark::State& state) {
+  using namespace gemmi;
+  Structure st = read_pdb_file(path);
+  while (state.KeepRunning()) {
+    bool has_hydr = has_hydrogen_with_cra(st);
+    benchmark::DoNotOptimize(has_hydr);
+  }
+}
+
+static void has_hydrogen3(benchmark::State& state) {
+  using namespace gemmi;
+  Structure st = read_pdb_file(path);
+  while (state.KeepRunning()) {
+    bool has_hydr = has_hydrogen_with_selection(st);
+    benchmark::DoNotOptimize(has_hydr);
+  }
+}
+
 int main(int argc, char** argv) {
   if (argc < 2) {
     printf("Call it with path to a pdb file as an argument.\n");
@@ -106,6 +157,9 @@ int main(int argc, char** argv) {
                                neighbor_search_for_each);
   benchmark::RegisterBenchmark("calculate_box", calculate_box);
   benchmark::RegisterBenchmark("fractional_box", fractional_box);
+  benchmark::RegisterBenchmark("has_hydrogen1", has_hydrogen1);
+  benchmark::RegisterBenchmark("has_hydrogen2", has_hydrogen2);
+  benchmark::RegisterBenchmark("has_hydrogen3", has_hydrogen3);
   benchmark::Initialize(&argc, argv);
   benchmark::RunSpecifiedBenchmarks();
 }
