@@ -12,6 +12,7 @@
 #include "to_mmcif.hpp"  // for write_struct_conn
 #include "sprintf.hpp"   // for to_str, to_str_prec
 #include "calculate.hpp" // for find_best_plane
+#include "select.hpp"    // for count_atom_sites
 
 namespace gemmi {
 
@@ -46,7 +47,8 @@ inline std::string get_ccp4_mod_id(const std::vector<std::string>& mods) {
   return ".";
 }
 
-inline cif::Block prepare_crd(const Structure& st, const Topo& topo) {
+inline cif::Block prepare_crd(const Structure& st, const Topo& topo,
+                              HydrogenChange h_change) {
   auto e_id = st.info.find("_entry.id");
   std::string id = cif::quote(e_id != st.info.end() ? e_id->second : st.name);
   cif::Block block("structure_" + id);
@@ -66,6 +68,14 @@ inline cif::Block prepare_crd(const Structure& st, const Topo& topo) {
     items.emplace_back("_audit.creation_date", initial_date->second);
   items.emplace_back("_software.name", "gemmi");
 
+  // this corresponds to Refmac keyword "make hydr"
+  const char* hydr = "A";  // appropriate for ReAdd*
+  if (h_change == HydrogenChange::NoChange || h_change == HydrogenChange::Shift)
+    hydr = "Y";
+  else if (h_change == HydrogenChange::Remove)
+    hydr = "N";
+  items.emplace_back("_ccp4_refmac.hatom", hydr);
+
   items.emplace_back(cif::CommentArg{"############\n"
                                      "## ENTITY ##\n"
                                      "############"});
@@ -156,12 +166,12 @@ inline cif::Block prepare_crd(const Structure& st, const Topo& topo) {
       "label_alt_id",
       "label_comp_id",
       "label_asym_id",
-      "auth_seq_id",
-      //"pdbx_PDB_ins_code",
+      "auth_seq_id", // including insertion code (no pdbx_PDB_ins_code)
       "Cartn_x",
       "Cartn_y",
       "Cartn_z",
       "occupancy",
+      "ccp4_hd_mixture",  // tags[11]
       "B_iso_or_equiv",
       "type_symbol",
       "calc_flag",
@@ -181,9 +191,16 @@ inline cif::Block prepare_crd(const Structure& st, const Topo& topo) {
     for (const Topo::ResInfo& ri : chain_info.res_infos) {
       const ChemComp& cc = ri.chemcomp;
       const Residue& res = *ri.res;
-      std::string auth_seq_id = res.seqid.num.str();
+      std::string auth_seq_id = res.seqid.str();
       //std::string ins_code(1, res.icode != ' ' ? res.icode : '?');
       for (const Atom& a : res.atoms) {
+        // infer hd_mixture (which is rarely used)
+        if (a.element == El::D && &a != &res.atoms[0] &&
+            (&a - 1)->element == El::H && a.name == (&a - 1)->name) {
+          double hd_mixture = (&a - 1)->occ / a.occ;
+          *(vv.end() - atom_loop.tags.size() + 11) = to_str(hd_mixture);
+          continue;
+        }
         vv.emplace_back("ATOM");
         vv.emplace_back(std::to_string(a.serial));
         vv.emplace_back(a.name);
@@ -191,11 +208,11 @@ inline cif::Block prepare_crd(const Structure& st, const Topo& topo) {
         vv.emplace_back(res.name);
         vv.emplace_back(cif::quote(chain_info.chain_ref.name));
         vv.emplace_back(auth_seq_id);
-        //vv.emplace_back(ins_code);
         vv.emplace_back(to_str(a.pos.x));
         vv.emplace_back(to_str(a.pos.y));
         vv.emplace_back(to_str(a.pos.z));
         vv.emplace_back(to_str(a.occ));
+        vv.emplace_back("0");  // hd_mixture
         vv.emplace_back(to_str(a.b_iso));
         vv.emplace_back(a.element.uname());
         vv.emplace_back(refmac_calc_flag(a));
