@@ -107,24 +107,24 @@ std::complex<T> lerp_(std::complex<T> a, std::complex<T> b, double t) {
   return a + (b - a) * (T) t;
 }
 
-// Catmull–Rom spline interpolation CINT_u from:
-// https://en.wikipedia.org/wiki/Cubic_Hermite_spline
-// The same as (24) in https://journals.iucr.org/d/issues/2018/06/00/ic5103/
+/// Catmull–Rom spline interpolation. CINT_u from:
+/// https://en.wikipedia.org/wiki/Cubic_Hermite_spline
+/// The same as (24) in https://journals.iucr.org/d/issues/2018/06/00/ic5103/
 inline double cubic_interpolation(double u, double a, double b, double c, double d) {
   //return 0.5 * u * (u * (u * (3*b - 3*c + d - a) + (2*a - 5*b + 4*c - d)) + (c - a)) + b;
-  // equivalent form that is faster (on my computer with GCC):
+  // equivalent form that is faster on my computer:
   return -0.5 * (a * u * ((u-2)*u + 1) - b * ((3*u - 5) * u*u + 2) +
                  u * (c * ((3*u - 4) * u - 1) - d * (u-1) * u));
 }
 
-// df/du (from Wolfram Alpha)
+/// df/du (from Wolfram Alpha)
 inline double cubic_interpolation_der(double u, double a, double b, double c, double d) {
   return a * (-1.5*u*u + 2*u - 0.5) + c * (-4.5*u*u + 4*u + 0.5)
          + u * (4.5*b*u - 5*b + 1.5*d*u - d);
 }
 
-// Order of grid axis. Some Grid functionality works only with the XYZ order.
-// The values XYZ and XYZ are used only when the grid covers whole unit cell.
+/// Order of grid axis. Some Grid functionality works only with the XYZ order.
+/// The values XYZ and ZYX are used only when the grid covers whole unit cell.
 enum class AxisOrder : unsigned char {
   Unknown,
   XYZ,  // default, corresponds to CCP4 map with axis order XYZ,
@@ -132,6 +132,7 @@ enum class AxisOrder : unsigned char {
   ZYX   // fast Z (or L), may not be fully supported everywhere
 };
 
+/// The base of Grid classes that does not depend on stored data type.
 struct GridMeta {
   UnitCell unit_cell;
   const SpaceGroup* spacegroup = nullptr;
@@ -139,6 +140,7 @@ struct GridMeta {
   AxisOrder axis_order = AxisOrder::Unknown;
 
   size_t point_count() const { return (size_t)nu * nv * nw; }
+  /// u,v,w are not normalized here
   Fractional get_fractional(int u, int v, int w) const {
     return {u * (1.0 / nu), v * (1.0 / nv), w * (1.0 / nw)};
   }
@@ -172,13 +174,15 @@ struct GridMeta {
     return grid_ops;
   }
 
-  // Quick but unsafe. assumes (for efficiency) that 0 <= u < nu, etc.
+  /// Quick(est) index function, but works only if `0 <= u < nu`, etc.
   size_t index_q(int u, int v, int w) const {
     return size_t(w * nv + v) * nu + u;
   }
 
-  // Assumes (for efficiency) that -nu <= u < 2*nu, etc.
-  // Modifies arguments.
+  /// Faster than index_s(), but works only if `-nu <= u < 2*nu`, etc.
+  size_t index_n(int u, int v, int w) const { return index_n_ref(u, v, w); }
+
+  /// The same as index_n(), but modifies arguments.
   size_t index_n_ref(int& u, int& v, int& w) const {
     if (u >= nu) u -= nu; else if (u < 0) u += nu;
     if (v >= nv) v -= nv; else if (v < 0) v += nv;
@@ -186,10 +190,7 @@ struct GridMeta {
     return this->index_q(u, v, w);
   }
 
-  // Assumes (for efficiency) that -nu <= u < 2*nu, etc.
-  size_t index_n(int u, int v, int w) const { return index_n_ref(u, v, w); }
-
-  // Assumes (for efficiency) that -nu <= u < nu, etc.
+  /// Faster than index_n(), but works only if -nu <= u < nu, etc.
   size_t index_near_zero(int u, int v, int w) const {
     return this->index_q(u >= 0 ? u : u + nu,
                          v >= 0 ? v : v + nv,
@@ -197,8 +198,10 @@ struct GridMeta {
   }
 };
 
+/// A common subset of Grid and ReciprocalGrid.
 template<typename T>
 struct GridBase : GridMeta {
+  /// grid coordinates (modulo size) and a pointer to value
   struct Point {
     int u, v, w;
     T* value;
@@ -211,9 +214,9 @@ struct GridBase : GridMeta {
       fail("grid is empty");
   }
 
-  void set_size_without_checking(int u, int v, int w) {
-    nu = u, nv = v, nw = w;
-    data.resize((size_t)u * v * w);
+  void set_size_without_checking(int nu_, int nv_, int nw_) {
+    nu = nu_, nv = nv_, nw = nw_;
+    data.resize((size_t)nu_ * nv_ * nw_);
   }
 
   T get_value_q(int u, int v, int w) const { return data[index_q(u, v, w)]; }
@@ -267,8 +270,9 @@ struct GridBase : GridMeta {
   iterator end() { return {*this, data.size()}; }
 };
 
-// For simplicity, some operations work only if the grid covers whole unit cell
-// and axes u,v,w correspond to a,b,c in the unit cell.
+/// Real-space grid.
+/// For simplicity, some operations work only if the grid covers whole unit cell
+/// and axes u,v,w correspond to a,b,c in the unit cell.
 template<typename T=float>
 struct Grid : GridBase<T> {
   using Point = typename GridBase<T>::Point;
@@ -279,9 +283,10 @@ struct Grid : GridBase<T> {
   using GridBase<T>::spacegroup;
   using GridBase<T>::data;
 
-  // spacing between virtual planes, not between points
+  /// spacing between virtual planes, not between points
   double spacing[3] = {0., 0., 0.};
 
+  /// copy unit_cell, spacegroup, nu, nv, nw, axis_order and set spacing
   void copy_metadata_from(const GridMeta& g) {
     unit_cell = g.unit_cell;
     spacegroup = g.spacegroup;
@@ -292,6 +297,7 @@ struct Grid : GridBase<T> {
     calculate_spacing();
   }
 
+  /// set #spacing
   void calculate_spacing() {
     spacing[0] = 1.0 / (nu * unit_cell.ar);
     spacing[1] = 1.0 / (nv * unit_cell.br);
@@ -302,15 +308,15 @@ struct Grid : GridBase<T> {
     return std::min(std::min(spacing[0], spacing[1]), spacing[2]);
   }
 
-  void set_size_without_checking(int u, int v, int w) {
-    GridBase<T>::set_size_without_checking(u, v, w);
+  void set_size_without_checking(int nu_, int nv_, int nw_) {
+    GridBase<T>::set_size_without_checking(nu_, nv_, nw_);
     calculate_spacing();
     this->axis_order = AxisOrder::XYZ;
   }
 
-  void set_size(int u, int v, int w) {
-    check_grid_factors(spacegroup, {{u, v, w}});
-    set_size_without_checking(u, v, w);
+  void set_size(int nu_, int nv_, int nw_) {
+    check_grid_factors(spacegroup, {{nu_, nv_, nw_}});
+    set_size_without_checking(nu_, nv_, nw_);
   }
 
   // The resulting spacing can be smaller (if denser=true) or greater than arg.
@@ -341,16 +347,22 @@ struct Grid : GridBase<T> {
     set_size_from_spacing(approx_spacing, denser);
   }
 
-  // Safe but slower.
+  /// Returns index in data array for (u,v,w). Safe but slower than index_q().
   size_t index_s(int u, int v, int w) const {
     this->check_not_empty();
     return this->index_q(modulo(u, nu), modulo(v, nv), modulo(w, nw));
   }
 
+  /// returns `data[index_s(u, v, w)]`
   T get_value(int u, int v, int w) const {
     return data[index_s(u, v, w)];
   }
 
+  void set_value(int u, int v, int w, T x) {
+    data[index_s(u, v, w)] = x;
+  }
+
+  /// Point stores normalizes indices (not the original u,v,w).
   Point get_point(int u, int v, int w) {
     u = modulo(u, nu);
     v = modulo(v, nv);
@@ -363,11 +375,15 @@ struct Grid : GridBase<T> {
       fail("grid is not fully setup");
     return get_point(iround(f.x * nu), iround(f.y * nv), iround(f.z * nw));
   }
-
   Point get_nearest_point(const Position& pos) {
     return get_nearest_point(unit_cell.fractionalize(pos));
   }
 
+  size_t get_nearest_index(const Fractional& f) {
+    return index_s(iround(f.x * nu), iround(f.y * nv), iround(f.z * nw));
+  }
+
+  /// Point stores normalized indices, so fractional coordinates are in [0,1).
   Fractional point_to_fractional(const Point& p) const {
     return this->get_fractional(p.u, p.v, p.w);
   }
@@ -381,7 +397,8 @@ struct Grid : GridBase<T> {
     return x - f;
   }
 
-  // https://en.wikipedia.org/wiki/Trilinear_interpolation
+  /// https://en.wikipedia.org/wiki/Trilinear_interpolation
+  /// x,y,z are grid coordinates (x=1.5 is between 2nd and 3rd grid point).
   T interpolate_value(double x, double y, double z) const {
     this->check_not_empty();
     int u, v, w;
@@ -410,7 +427,8 @@ struct Grid : GridBase<T> {
     return interpolate_value(unit_cell.fractionalize(ctr));
   }
 
-  // https://en.wikipedia.org/wiki/Tricubic_interpolation
+  /// https://en.wikipedia.org/wiki/Tricubic_interpolation
+  /// x,y,z are grid coordinates (x=1.5 is between 2nd and 3rd grid point).
   double tricubic_interpolation(double x, double y, double z) const {
     std::array<std::array<std::array<T,4>,4>,4> copy;
     copy_4x4x4(x, y, z, copy);
@@ -429,7 +447,7 @@ struct Grid : GridBase<T> {
   double tricubic_interpolation(const Position& ctr) const {
     return tricubic_interpolation(unit_cell.fractionalize(ctr));
   }
-  // the same + derivatives df/dx, df/dy, df/dz
+  /// returns the same as above + derivatives df/dx, df/dy, df/dz
   std::array<double,4> tricubic_interpolation_der(double x, double y, double z) const {
     std::array<std::array<std::array<T,4>,4>,4> copy;
     copy_4x4x4(x, y, z, copy);
@@ -459,6 +477,7 @@ struct Grid : GridBase<T> {
     auto r = tricubic_interpolation_der(fctr.x * nu, fctr.y * nv, fctr.z * nw);
     return {r[0], r[1] * nu, r[2] * nv, r[3] * nw};
   }
+  /// @private
   void copy_4x4x4(double& x, double& y, double& z,
                   std::array<std::array<std::array<T,4>,4>,4>& copy) const {
     this->check_not_empty();
@@ -485,6 +504,16 @@ struct Grid : GridBase<T> {
           copy[i][j][k] = this->get_value_q(u_indices[i], v_indices[j], w_indices[k]);
   }
 
+  /// @param order 1=nearest, 2=linear, 3=cubic interpolation
+  T interpolate(const Fractional& f, int order) const {
+    switch (order) {
+      case 1: return *const_cast<Grid<T>*>(this)->get_nearest_point(f).value;
+      case 2: return interpolate_value(f);
+      case 3: return (T) tricubic_interpolation(f);
+    }
+    throw std::invalid_argument("interpolation \"order\" must 1, 2 or 3");
+  }
+
   void get_subarray(T* dest, std::array<int,3> start, std::array<int,3> shape) const {
     this->check_not_empty();
     if (this->axis_order != AxisOrder::XYZ)
@@ -537,8 +566,6 @@ struct Grid : GridBase<T> {
     }
   }
 
-  void set_value(int u, int v, int w, T x) { data[index_s(u, v, w)] = x; }
-
   template <bool UsePbc>
   void check_size_for_points_in_box(int& du, int& dv, int& dw,
                                     bool fail_on_too_large_radius) const {
@@ -557,8 +584,6 @@ struct Grid : GridBase<T> {
 
   template <bool UsePbc, typename Func>
   void do_use_points_in_box(Fractional fctr, int du, int dv, int dw, Func&& func) {
-    if (UsePbc)
-      fctr = fctr.wrap_to_unit();
     int u0 = iround(fctr.x * nu);
     int v0 = iround(fctr.y * nv);
     int w0 = iround(fctr.z * nw);
@@ -576,14 +601,23 @@ struct Grid : GridBase<T> {
       w_lo = std::max(w_lo, 0);
       w_hi = std::min(w_hi, nw - 1);
     }
-    for (int w = w_lo; w <= w_hi; ++w)
-      for (int v = v_lo; v <= v_hi; ++v)
+    const Position orth0(unit_cell.orth.mat.column_copy(0));
+    for (int w = w_lo; w <= w_hi; ++w) {
+      int w_ = UsePbc ? modulo(w, nw) : w;
+      double fw = w * (1.0 / nw);
+      for (int v = v_lo; v <= v_hi; ++v) {
+        int v_ = UsePbc ? modulo(v, nv) : v;
+        double fv = v * (1.0 / nv);
+        size_t idx0 = this->index_q(0, v_, w_);
+        Position delta0 = unit_cell.orthogonalize_difference(fctr - Fractional(0., fv, fw));
         for (int u = u_lo; u <= u_hi; ++u) {
-          Fractional fdelta = fctr - this->get_fractional(u, v, w);
-          Position delta = unit_cell.orthogonalize_difference(fdelta);
-          size_t idx = UsePbc ? this->index_n(u, v, w) : this->index_q(u, v, w);
-          func(data[idx], delta);
+          int u_ = UsePbc ? modulo(u, nu) : u;
+          double fu = u * (1.0 / nu);
+          Position delta = delta0 - orth0 * fu;
+          func(data[idx0 + u_], delta, u, v, w);
         }
+      }
+    }
   }
 
   template <bool UsePbc, typename Func>
@@ -600,10 +634,10 @@ struct Grid : GridBase<T> {
     int dv = (int) std::ceil(radius / spacing[1]);
     int dw = (int) std::ceil(radius / spacing[2]);
     use_points_in_box<UsePbc>(fctr_, du, dv, dw,
-                      [&](T& point, const Position& delta) {
+                      [&](T& ref, const Position& delta, int, int, int) {
                         double d2 = delta.length_sq();
                         if (d2 < radius * radius)
-                          func(point, d2);
+                          func(ref, d2);
                       },
                       fail_on_too_large_radius);
   }
@@ -611,9 +645,9 @@ struct Grid : GridBase<T> {
   void set_points_around(const Position& ctr, double radius, T value, bool use_pbc=true) {
     Fractional fctr = unit_cell.fractionalize(ctr);
     if (use_pbc)
-      use_points_around<true>(fctr, radius, [&](T& point, double) { point = value; });
+      use_points_around<true>(fctr, radius, [&](T& ref, double) { ref = value; });
     else
-      use_points_around<false>(fctr, radius, [&](T& point, double) { point = value; });
+      use_points_around<false>(fctr, radius, [&](T& ref, double) { ref = value; });
   }
 
   void change_values(T old_value, T new_value) {
@@ -622,6 +656,14 @@ struct Grid : GridBase<T> {
         d = new_value;
   }
 
+  /// Use \par func to reduce values of all symmetry mates of each
+  /// grid point, then assign the result to all the points.
+  /// \par func takes two values and returns a value.
+  template<typename Func>
+  void symmetrize(Func func) {
+    symmetrize_using_ops(this->get_scaled_ops_except_id(), func);
+  }
+
   template<typename Func>
   void symmetrize_using_ops(const std::vector<GridOp>& ops, Func func) {
     if (ops.empty())
@@ -655,14 +697,7 @@ struct Grid : GridBase<T> {
     assert(idx == data.size());
   }
 
-  // Use provided function to reduce values of all symmetry mates of each
-  // grid point, then assign the result to all the points.
-  template<typename Func>
-  void symmetrize(Func func) {
-    symmetrize_using_ops(this->get_scaled_ops_except_id(), func);
-  }
-
-  // two most common symmetrize functions
+  // most common symmetrize functions
   void symmetrize_min() {
     symmetrize([](T a, T b) { return (a < b || !(b == b)) ? a : b; });
   }
@@ -672,21 +707,22 @@ struct Grid : GridBase<T> {
   void symmetrize_abs_max() {
     symmetrize([](T a, T b) { return (std::abs(a) > std::abs(b) || !(b == b)) ? a : b; });
   }
-  // multiplies grid points on special position
+  /// multiplies grid points on special position
   void symmetrize_sum() {
     symmetrize([](T a, T b) { return a + b; });
   }
-
-  T interpolate(const Fractional& f, int order) const {
-    switch (order) {
-      case 1: return *const_cast<Grid<T>*>(this)->get_nearest_point(f).value;
-      case 2: return interpolate_value(f);
-      case 3: return (T) tricubic_interpolation(f);
-    }
-    throw std::invalid_argument("interpolation \"order\" must 1, 2 or 3");
+  void symmetrize_nondefault(T default_) {
+    symmetrize([default_](T a, T b) { return impl::is_same(a, default_) ? b : a; });
   }
 
-  // order 1=nearest, 2=linear, 3=cubic interpolation
+  /// scale the data to get mean == 0 and rmsd == 1 (doesn't work for T=complex)
+  void normalize() {
+    DataStats stats = calculate_data_statistics(data);
+    for (T& x : data)
+      x = static_cast<T>((x - stats.dmean) / stats.rms);
+  }
+
+  // TODO: can it be replaced with interpolate_grid(dest, src, Transform(), order)?
   void resample_to(Grid<T>& dest, int order) const {
     dest.check_not_empty();
     int idx = 0;
@@ -711,13 +747,5 @@ Correlation calculate_correlation(const GridBase<T>& a, const GridBase<T>& b) {
   return corr;
 }
 
-// scale the data to get mean == 0 and rmsd == 1
-template<typename T>
-void normalize_grid(Grid<T>& grid) {
-  DataStats stats = calculate_data_statistics(grid.data);
-  for (auto i = grid.data.begin(); i != grid.data.end(); ++i)
-    *i = static_cast<T>((*i - stats.dmean) / stats.rms);
-}
-
 } // namespace gemmi
 #endif
