@@ -32,17 +32,18 @@ inline void add_distinct_altlocs(const Residue& res, std::string& altlocs) {
 struct ChemLink {
   enum class Group {
     // _chem_link.group_comp_N is one of:
-    // "peptide", "P-peptide", "M-peptide", "pyranose", "DNA/RNA" or null
+    // "peptide", "P-peptide", "M-peptide", "pyranose", "ketopyranose", "DNA/RNA" or null
     // (we ignore "polymer")
-    Peptide, PPeptide, MPeptide, Pyranose, DnaRna, Null
+    Peptide, PPeptide, MPeptide, Pyranose, Ketopyranose, DnaRna, Null
   };
   struct Side {
     std::string comp;
     std::string mod;
     Group group;
     bool matches_group(Group res) const {
-      return res == group ||
-             (group == Group::Peptide && (int) res <= (int) Group::MPeptide);
+      return group != Group::Null &&
+             (res == group ||
+              (group == Group::Peptide && (int) res <= (int) Group::MPeptide));
     }
     int specificity() const {
       if (!comp.empty())
@@ -67,6 +68,7 @@ struct ChemLink {
         case ialpha4_id("p-pe"): return Group::PPeptide;
         case ialpha4_id("m-pe"): return Group::MPeptide;
         case ialpha4_id("pyra"): return Group::Pyranose;
+        case ialpha4_id("keto"): return Group::Ketopyranose;
         case ialpha4_id("dna/"): return Group::DnaRna;
       }
     }
@@ -79,6 +81,7 @@ struct ChemLink {
       case Group::PPeptide: return "P-peptide";
       case Group::MPeptide: return "M-peptide";
       case Group::Pyranose: return "pyranose";
+      case Group::Ketopyranose: return "ketopyranose";
       case Group::DnaRna: return "DNA/RNA";
       case Group::Null: return ".";
     }
@@ -97,6 +100,7 @@ struct ChemLink {
       case ResidueInfo::BUF:     return Group::Null;
       case ResidueInfo::HOH:     return Group::Null;
       case ResidueInfo::PYR:     return Group::Pyranose;
+      case ResidueInfo::KET:     return Group::Ketopyranose;
       case ResidueInfo::ELS:     return Group::Null;
     }
     unreachable();
@@ -228,6 +232,7 @@ inline ResidueInfo::Kind chemcomp_group_to_kind(const std::string& group) {
       case ialpha4_id("dna"): return ResidueInfo::DNA;
       case ialpha4_id("rna"): return ResidueInfo::RNA;
       case ialpha4_id("pyra"): return ResidueInfo::PYR;
+      case ialpha4_id("keto"): return ResidueInfo::KET;
     }
   }
   return ResidueInfo::UNKNOWN;
@@ -265,7 +270,7 @@ inline void insert_chemlinks(const cif::Document& doc,
       link.side2.group = ChemLink::read_group(row[7]);
       const cif::Block* block = doc.find_block("link_" + link.id);
       if (!block)
-        fail("inconsisted data_link_list");
+        fail("inconsistent data_link_list");
       link.rt = read_link_restraints(*block);
       links.emplace(link.id, link);
     }
@@ -339,7 +344,7 @@ inline void insert_chemmods(const cif::Document& doc,
       mod.group_id = row.str(3);
       const cif::Block* block = doc.find_block("mod_" + mod.id);
       if (!block)
-        fail("inconsisted data_mod_list");
+        fail("inconsistent data_mod_list");
       for (auto ra : const_cast<cif::Block*>(block)->find("_chem_mod_atom.",
                                   {"function", "atom_id", "new_atom_id",
                                    "new_type_symbol", "new_type_energy",
@@ -360,7 +365,7 @@ inline void ChemMod::apply_to(ChemComp& chemcomp) const {
     auto it = chemcomp.find_atom(mod.old_id);
     switch (mod.func) {
       case 'a':
-        if (chemcomp.find_atom(mod.new_id) == chemcomp.atoms.end())
+        if (!chemcomp.has_atom(mod.new_id))
           chemcomp.atoms.push_back({mod.new_id, mod.el,
                                     std::isnan(mod.charge) ? mod.charge : 0,
                                     mod.chem_type});
@@ -537,14 +542,39 @@ inline void ChemMod::apply_to(ChemComp& chemcomp) const {
     }
 }
 
+struct EnerLib {
+  struct Atom {
+    Element element;
+    char hb_type;
+    double vdw_radius;
+    double vdwh_radius;
+    double ion_radius;
+    int valency;
+    int sp;
+  };
+
+  EnerLib() {}
+  EnerLib(const cif::Document& doc) {
+    cif::Block& block = const_cast<cif::Block&>(doc.blocks[0]);
+    for (const auto& row : block.find("_lib_atom.",
+                    {"type", "hb_type", "vdw_radius", "vdwh_radius",
+                     "ion_radius", "element", "valency", "sp"}))
+      atoms.emplace(row[0], Atom{Element(row[5]), row[1][0], cif::as_number(row[2]),
+                                 cif::as_number(row[3]), cif::as_number(row[4]),
+                                 cif::as_int(row[6], -1), cif::as_int(row[7], -1)});
+  }
+  std::map<std::string, Atom> atoms; // type->Atom
+};
+
 struct MonLib {
   cif::Document mon_lib_list;
+  EnerLib ener_lib;
   std::map<std::string, ChemComp> monomers;
   std::map<std::string, ChemLink> links;
   std::map<std::string, ChemMod> modifications;
   std::map<std::string, ResidueInfo> residue_infos;
 
-  const ChemLink* find_link(const std::string& link_id) const {
+  const ChemLink* get_link(const std::string& link_id) const {
     auto link = links.find(link_id);
     return link != links.end() ? &link->second : nullptr;
   }
@@ -556,26 +586,49 @@ struct MonLib {
     auto resinfo = residue_infos.find(name);
     return resinfo != residue_infos.end() ? &resinfo->second : nullptr;
   }
-  const ChemLink* match_link(
-      const std::string& comp1, const std::string& atom1,
-      const std::string& comp2, const std::string& atom2) const {
+
+  // Returns the most specific link and a flag that is true
+  // if the order is comp2-comp1 in the link definition.
+  std::pair<const ChemLink*, bool>
+  match_link(const Residue& res1, const std::string& atom1,
+             const Residue& res2, const std::string& atom2,
+             char alt) const {
+    const ChemLink* best_link = nullptr;
+    int best_score = -1;
+    bool inverted = false;
     for (auto& ml : links) {
       const ChemLink& link = ml.second;
+      if (link.rt.bonds.empty())
+        continue;
       // for now we don't have link definitions with >1 bonds
-      if (link.rt.bonds.size() == 1) {
-        const Restraints::Bond& bond = link.rt.bonds[0];
-        if (bond.id1.atom == atom1 && bond.id2.atom == atom2 &&
-            link_side_matches_residue(link.side1, comp1) &&
-            link_side_matches_residue(link.side2, comp2))
-          return &link;
+      const Restraints::Bond& bond = link.rt.bonds[0];
+      if (bond.id1.atom == atom1 && bond.id2.atom == atom2 &&
+          link_side_matches_residue(link.side1, res1.name) &&
+          link_side_matches_residue(link.side2, res2.name)) {
+        int score = link.calculate_score(res1, &res2, alt);
+        if (score > best_score) {
+          best_link = &link;
+          best_score = score;
+          inverted = false;
+        }
+      }
+      if (bond.id1.atom == atom2 && bond.id2.atom == atom1 &&
+          link_side_matches_residue(link.side1, res2.name) &&
+          link_side_matches_residue(link.side2, res1.name)) {
+        int score = link.calculate_score(res2, &res1, alt);
+        if (score > best_score) {
+          best_link = &link;
+          best_score = score;
+          inverted = true;
+        }
       }
     }
-    return nullptr;
+    return {best_link, inverted};
   }
 
   void ensure_unique_link_name(std::string& name) const {
     size_t orig_len = name.size();
-    for (int n = 0; find_link(name) != nullptr; ++n)
+    for (int n = 0; get_link(name) != nullptr; ++n)
       name.replace(orig_len, name.size(), std::to_string(n));
   }
 
@@ -617,7 +670,7 @@ struct MonLib {
     path += code;
     // On Windows several names are reserved (CON, PRN, AUX, ...), see
     // https://docs.microsoft.com/en-us/windows/win32/fileio/naming-a-file
-    // The workaround in CCP4 monomer libary is to use CON_CON.cif, etc.
+    // The workaround in CCP4 monomer library is to use CON_CON.cif, etc.
     if (code.size() == 3)
       switch (ialpha3_id(code.c_str())) {
         case ialpha3_id("AUX"):
@@ -632,14 +685,36 @@ struct MonLib {
     return path;
   }
 
-  void read_monomer_cif(const std::string& path, read_cif_func read_cif) {
-    mon_lib_list = (*read_cif)(path);
+  void read_monomer_cif(const std::string& path_, read_cif_func read_cif) {
+    mon_lib_list = (*read_cif)(path_);
     for (const cif::Block& block : mon_lib_list.blocks)
       add_monomer_if_present(block);
     insert_chemlinks(mon_lib_list, links);
     insert_chemmods(mon_lib_list, modifications);
     insert_comp_list(mon_lib_list, residue_infos);
   }
+
+  /// Searches data from _lib_atom in ener_lib.cif.
+  /// If chem_type is not in the library uses element name as chem_type.
+  double find_radius(const const_CRA& cra, bool use_ion) const {
+    double r = 1.7;
+    auto it = ener_lib.atoms.end();
+    auto cc = monomers.find(cra.residue->name);
+    if (cc != monomers.end()) {
+      auto cc_atom = cc->second.find_atom(cra.atom->name);
+      if (cc_atom != cc->second.atoms.end())
+        it = ener_lib.atoms.find(cc_atom->chem_type);
+    }
+    if (it == ener_lib.atoms.end())
+      it = ener_lib.atoms.find(cra.atom->element.uname());
+    if (it != ener_lib.atoms.end()) {
+      if (use_ion && !std::isnan(it->second.ion_radius))
+        r = it->second.ion_radius;
+      else
+        r = it->second.vdw_radius;
+    }
+    return r;
+  }
 };
 
 inline MonLib read_monomer_lib(std::string monomer_dir,
@@ -656,12 +731,17 @@ inline MonLib read_monomer_lib(std::string monomer_dir,
     monlib.read_monomer_cif(libin, read_cif);
   monlib.read_monomer_cif(monomer_dir + "list/mon_lib_list.cif", read_cif);
   std::string error;
+  try {
+    monlib.ener_lib = EnerLib((*read_cif)(monomer_dir + "/ener_lib.cif"));
+  } catch (std::runtime_error& err) {
+    error.append("ener_lib.cif could not be read: ").append(err.what()).append(".\n");
+  }
   for (const std::string& name : resnames) {
     try {
       cif::Document doc = (*read_cif)(monomer_dir + MonLib::relative_monomer_path(name));
       auto cc = make_chemcomp_from_cif(name, doc);
       monlib.monomers.emplace(name, std::move(cc));
-    } catch(std::runtime_error& err) {
+    } catch (std::runtime_error& err) {
       if (!ignore_missing)
         error += "The monomer " + name + " could not be read: " + err.what() + ".\n";
     }
