@@ -183,11 +183,8 @@ struct Mtz {
   FILE* warnings = nullptr;
 
   explicit Mtz(bool with_base=false) {
-    if (with_base) {
-      datasets.push_back({0, "HKL_base", "HKL_base", "HKL_base", cell, 0.});
-      for (int i = 0; i != 3; ++i)
-        add_column(std::string(1, "HKL"[i]), 'H', 0, i);
-    }
+    if (with_base)
+      add_base();
   }
   Mtz(Mtz&& o) noexcept { *this = std::move(o); }
   Mtz& operator=(Mtz&& o) noexcept {
@@ -220,6 +217,12 @@ struct Mtz {
   Mtz(Mtz const&) = delete;
   Mtz& operator=(Mtz const&) = delete;
 
+  void add_base() {
+    datasets.push_back({0, "HKL_base", "HKL_base", "HKL_base", cell, 0.});
+    for (int i = 0; i != 3; ++i)
+      add_column(std::string(1, "HKL"[i]), 'H', 0, i, false);
+  }
+
   // Functions to use after MTZ headers (and data) is read.
 
   double resolution_high() const { return std::sqrt(1.0 / max_1_d2); }
@@ -244,16 +247,24 @@ struct Mtz {
   }
 
   UnitCell get_average_cell_from_batch_headers(double* rmsd) const {
+    if (rmsd)
+      for (int i = 0; i < 6; ++i)
+        rmsd[i] = 0.;
     double avg[6] = {0., 0., 0., 0., 0., 0.};
     for (const Batch& batch : batches)
-      for (int i = 0; i < 6; ++i)
+      for (int i = 0; i < 6; ++i) {
+        // if batch headers are not set correctly, return global cell
+        if (batch.floats[i] <= 0)
+          return cell;
         avg[i] += batch.floats[i];
+      }
+    if (avg[0] <= 0 || avg[1] <= 0 || avg[2] <= 0 ||
+        avg[3] <= 0 || avg[4] <= 0 || avg[5] <= 0)
+      return UnitCell();
     size_t n = batches.size();
     for (int i = 0; i < 6; ++i)
       avg[i] /= n;
     if (rmsd) {
-      for (int i = 0; i < 6; ++i)
-        rmsd[i] = 0.;
       for (const Batch& batch : batches)
         for (int i = 0; i < 6; ++i)
           rmsd[i] += sq(avg[i] - batch.floats[i]);
@@ -804,20 +815,20 @@ struct Mtz {
   }
 
   // (for merged MTZ only) change HKL to ASU equivalent, adjust phases
-  void ensure_asu() {
+  void ensure_asu(bool tnt_asu=false) {
     if (!is_merged())
       fail("Mtz::ensure_asu() is for merged MTZ only");
     if (!spacegroup)
       return;
     GroupOps gops = spacegroup->operations();
-    ReciprocalAsu asu(spacegroup);
+    ReciprocalAsu asu(spacegroup, tnt_asu);
     std::vector<int> phase_columns = positions_of_columns_with_type('P');
     std::vector<int> abcd_columns = positions_of_columns_with_type('A');
     std::vector<int> dano_columns = positions_of_columns_with_type('D');
     std::vector<std::pair<int,int>> plus_minus_columns = positions_of_plus_minus_columns();
     bool no_special_columns = phase_columns.empty() && abcd_columns.empty() &&
                               plus_minus_columns.empty() && dano_columns.empty();
-    bool centric = no_special_columns || gops.is_centric();
+    bool centric = no_special_columns || gops.is_centrosymmetric();
     for (size_t n = 0; n < data.size(); n += columns.size()) {
       Miller hkl = get_hkl(n);
       if (asu.is_in(hkl))
@@ -921,7 +932,7 @@ struct Mtz {
   }
 
   Column& add_column(const std::string& label, char type,
-                     int dataset_id=-1, int pos=-1, bool expand_data=false) {
+                     int dataset_id, int pos, bool expand_data) {
     if (datasets.empty())
       fail("No datasets.");
     if (dataset_id < 0)
@@ -1002,8 +1013,7 @@ struct Mtz {
               src_mtz->data[*src * src_stride + src_col.idx + i];
           ++dst;
           ++src;
-        } else if (std::tie(dst_hkl[0], dst_hkl[1], dst_hkl[2]) <
-                   std::tie(src_hkl[0], src_hkl[1], src_hkl[2])) {
+        } else if (dst_hkl < src_hkl) {
           ++dst;
         } else {
           ++src;
@@ -1040,7 +1050,7 @@ struct Mtz {
         col_idx += 1 + (int)trailing_cols.size();
     }
     for (int i = 0; i <= (int) trailing_cols.size(); ++i)
-      add_column("", ' ', -1, dest_idx + i);
+      add_column("", ' ', -1, dest_idx + i, false);
     expand_data_rows(1 + trailing_cols.size(), dest_idx);
     // copy the data
     const Column& src_col_now = col_idx < 0 ? src_col : columns[col_idx];
@@ -1059,6 +1069,22 @@ struct Mtz {
     data.resize(columns.size() * nreflections);
   }
 
+  template <typename Func>
+  void remove_rows_if(Func condition) {
+    if (!has_data())
+      fail("No data.");
+    auto out = data.begin();
+    size_t width = columns.size();
+    for (auto r = data.begin(); r < data.end(); r += width)
+      if (!condition(&*r)) {
+        if (r != out)
+          std::copy(r, r + width, out);
+        out += width;
+      }
+    data.erase(out, data.end());
+    nreflections = int(data.size() / width);
+  }
+
   void expand_data_rows(size_t added, int pos_=-1) {
     size_t old_row_size = columns.size() - added;
     if (data.size() != old_row_size * nreflections)
