@@ -152,10 +152,12 @@ void add_cif_atoms(const Structure& st, cif::Block& block, bool use_group_pdb) {
     atom_loop.tags.emplace(atom_loop.tags.begin(), "_atom_site.group_PDB");
   bool has_calc_flag = false;
   bool has_tls_group_id = false;
+  size_t atom_site_count = 0;
   for (const Model& model : st.models)
     for (const Chain& chain : model.chains)
       for (const Residue& res : chain.residues)
         for (const Atom& atom : res.atoms) {
+          ++atom_site_count;
           if (atom.calc_flag != CalcFlag::NotSet)
             has_calc_flag = true;
           if (atom.tls_group_id >= 0)
@@ -167,7 +169,7 @@ void add_cif_atoms(const Structure& st, cif::Block& block, bool use_group_pdb) {
     atom_loop.tags.emplace_back("_atom_site.pdbx_tls_group_id");
 
   std::vector<std::string>& vv = atom_loop.values;
-  vv.reserve(count_atom_sites(st) * atom_loop.tags.size());
+  vv.reserve(atom_site_count * atom_loop.tags.size());
   std::vector<std::pair<int, const Atom*>> aniso;
   int serial = 0;
   for (const Model& model : st.models) {
@@ -863,12 +865,13 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
     cif::Loop& asym_loop = block.init_mmcif_loop("_struct_asym.",
                                                  {"id", "entity_id"});
     for (const Chain& chain : st.models[0].chains)
-      for (ConstResidueSpan& sub : chain.subchains())
-        if (!sub.subchain_id().empty()) {
-          const Entity* ent = st.get_entity_of(sub);
-          asym_loop.add_row({sub.subchain_id(),
-                             (ent ? impl::qchain(ent->name) : "?")});
+      for (ConstResidueSpan& sub : chain.subchains()) {
+        const std::string& sub_id = sub.subchain_id();
+        if (!sub_id.empty()) {
+          const Entity* ent = find_entity_of_subchain(sub_id, st.entities);
+          asym_loop.add_row({sub_id, (ent ? impl::qchain(ent->name) : "?")});
         }
+      }
   }
 
   if (groups.origx) { // _database_PDB_matrix (ORIGX)
