@@ -2,7 +2,6 @@
 
 #include "gemmi/chemcomp.hpp"    // for ChemComp
 #include "gemmi/to_chemcomp.hpp" // for add_chemcomp_to_block
-#include "gemmi/tostr.hpp"       // for tostr
 
 #include "common.h"
 #include <pybind11/stl.h>
@@ -70,7 +69,7 @@ void add_chemcomp(py::module& m) {
          py::arg("res1"), py::arg("res2"), py::arg("altloc"),
          py::return_value_policy::reference)
     .def("__repr__", [](const Restraints::AtomId& self) {
-        return tostr("<gemmi.Restraints.AtomId ", self.comp, ' ', self.atom, '>');
+        return cat("<gemmi.Restraints.AtomId ", self.comp, ' ', self.atom, '>');
     });
   restraintsbond
     .def_readwrite("id1", &Restraints::Bond::id1)
@@ -152,11 +151,15 @@ void add_chemcomp(py::module& m) {
     .def("is_hydrogen", &ChemComp::Atom::is_hydrogen)
     ;
   chemcomp
-    .def_readonly("name", &ChemComp::name)
-    .def_readonly("group", &ChemComp::group)
+    .def_readwrite("name", &ChemComp::name)
+    .def_readwrite("group", &ChemComp::group)
     .def_readonly("atoms", &ChemComp::atoms)
     .def_readonly("rt", &ChemComp::rt)
     .def("get_atom", &ChemComp::get_atom)
+    .def("find_atom", [](ChemComp& self, const std::string& atom_id) {
+        auto it = self.find_atom(atom_id);
+        return it != self.atoms.end() ? &*it : nullptr;
+    }, py::return_value_policy::reference_internal)
     .def("remove_hydrogens", &ChemComp::remove_hydrogens)
     ;
   m.def("make_chemcomp_from_block", &make_chemcomp_from_block);
