@@ -160,6 +160,7 @@ void add_grid_interpolation(py::class_<Grid<T>, GridBase<T>>& grid) {
          (T (Gr::*)(const Fractional&) const) &Gr::interpolate_value)
     .def("interpolate_value",
          (T (Gr::*)(const Position&) const) &Gr::interpolate_value)
+    // TODO: find a better name for this func, perhaps interpolate_array?
     .def("interpolate_values",
          [](const Gr& self, py::array_t<T> arr, const Transform& tr, int order) {
         auto r = arr.template mutable_unchecked<3>();
@@ -201,7 +202,8 @@ void add_grid(py::module& m) {
       return py::make_tuple(self.nu, self.nv, self.nw);
     });
 
-  add_grid_base<int8_t>(m, "Int8GridBase");
+  add_grid_base<int8_t>(m, "Int8GridBase")
+    .def("get_nonzero_extent", &get_nonzero_extent<int8_t>);
   add_grid_common<int8_t>(m, "Int8Grid");
 
   add_grid_base<float>(m, "FloatGridBase")
@@ -210,7 +212,7 @@ void add_grid(py::module& m) {
     ;
   auto grid_float = add_grid_common<float>(m, "FloatGrid");
   add_grid_interpolation<float>(grid_float);
-  grid_float.def("normalize", &normalize_grid<float>);
+  grid_float.def("normalize", &Grid<float>::normalize);
   grid_float.def("add_soft_edge_to_mask", &add_soft_edge_to_mask<float>);
 
   add_grid_base<std::complex<float>>(m, "ComplexGridBase");
@@ -235,9 +237,11 @@ void add_grid(py::module& m) {
     .def("put_mask_on_float_grid", &SolventMasker::put_mask_on_grid<float>)
     .def("set_to_zero", &SolventMasker::set_to_zero)
     ;
+  m.def("interpolate_grid", &interpolate_grid<float>,
+        py::arg("dest"), py::arg("src"), py::arg("tr"), py::arg("order")=2);
   m.def("interpolate_grid_of_aligned_model2", &interpolate_grid_of_aligned_model2<float>,
         py::arg("dest"), py::arg("src"), py::arg("tr"),
-        py::arg("dest_model"), py::arg("radius"));
+        py::arg("dest_model"), py::arg("radius"), py::arg("order")=2);
 
 
   // from blob.hpp
