@@ -187,6 +187,7 @@ void add_unitcell(py::module& m) {
     .def_readonly("vec", &Transform::vec)
     .def("inverse", &Transform::inverse)
     .def("apply", &Transform::apply)
+    .def("combine", &Transform::combine)
     .def("is_identity", &Transform::is_identity)
     .def("approx", &Transform::approx, py::arg("other"), py::arg("epsilon"));
 
@@ -287,6 +288,7 @@ void add_unitcell(py::module& m) {
     .def("calculate_u_eq", &UnitCell::calculate_u_eq)
     .def("fractionalize", &UnitCell::fractionalize)
     .def("orthogonalize", &UnitCell::orthogonalize)
+    .def("op_as_transform", &UnitCell::op_as_transform)
     .def("volume_per_image", &UnitCell::volume_per_image)
     .def("find_nearest_image", &UnitCell::find_nearest_image,
          py::arg("ref"), py::arg("pos"), py::arg("asu")=Asu::Any)
