@@ -198,8 +198,8 @@ void write_asudata_to_mtz(const gemmi::AsuData<std::complex<Real>>& asu_data,
   output_mtz->set_cell_for_all(asu_data.unit_cell());
   output_mtz->spacegroup = asu_data.spacegroup();
   output_mtz->add_dataset("calculated");
-  output_mtz->add_column(file.f_label, 'F');
-  output_mtz->add_column(file.phi_label, 'P');
+  output_mtz->add_column(file.f_label, 'F', -1, -1, false);
+  output_mtz->add_column(file.phi_label, 'P', -1, -1, false);
   output_mtz->title = "Fcalc generated by gemmi";
   output_mtz->nreflections = (int) asu_data.v.size();
   output_mtz->data.reserve(5 * asu_data.v.size());
@@ -235,7 +235,7 @@ void process_with_fft(const gemmi::Structure& st,
   if (map_file) {
     gemmi::Ccp4<Real> ccp4;
     ccp4.grid = dencalc.grid;
-    ccp4.update_ccp4_header(2, true);
+    ccp4.update_ccp4_header(2);
     ccp4.write_ccp4_map(map_file);
   }
 
